#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import glob
import logging
import os
import pathlib
import shutil
import sys
import tarfile
from typing import Dict

from .interfaces import ConveyorIF

"""
Deliver weblog and standard capability (i.e. standard calibration or imaging) results to QA area for analyst access
"""

vlapipe_id = 6000


class DeliveryConveyor(ConveyorIF):
    def __init__(self, settings: Dict[str, str], action: str):
        self.logger = logging.getLogger("conveyor")
        self.action = action
        self.settings = settings

    def delivery(self):
        if "Calibration" in self.action:
            self.logger.info("Moving subdirectories to QA2 area!")
            qa_path = self.settings["qa_delivery_area"] + "/" + self.settings["destination_subdir"]
            self.cache_weblog()
            self.move_subdirectories_to_qa_area(qa_path)
        elif "Image" in self.action:
            self.logger.info("Moving subdirectories to Image QA area!")
            qa_path = self.settings["image_qa_area"] + "/" + self.settings["destination_subdir"]
            self.cache_weblog()
            self.move_subdirectories_to_qa_area(qa_path)

    def cache_weblog(self):
        self.logger.info("Caching Weblog!")
        weblog_location = self.settings["weblog_cache_area"] + "/" + self.settings["destination_subdir"]

        # make new cache directory with name matching workspaces parent directory and unzip weblog into it
        os.makedirs(weblog_location)
        os.chmod(weblog_location, 0o2770)
        location_path = self.settings["destination_dir"] + "/products"
        if pathlib.Path(location_path + "/weblog.tgz").exists():
            tar = tarfile.open(location_path + "/weblog.tgz")
            # extract to cache area
            tar.extractall(path=weblog_location)
            # extract in place
            tar.extractall(path=location_path)
            tar.close()
            self.create_qa_notes()
        elif self._get_pipeline_dir():
            self.logger.info("WARNING! No weblog.tgz, copying pipeline directory instead.")
            self.create_qa_notes()

            pipeline_dir = glob.glob(location_path + "/pipeline-*")[0]
            shutil.copytree(pipeline_dir, weblog_location)
        else:
            self.logger.error("ERROR: Neither weblog directory or tar file found! Exiting.")
            sys.exit(1)

    def _get_pipeline_dir(self) -> bool:
        if not glob.glob(self.settings["destination_dir"] + "/products/pipeline-*"):
            return False
        return True

    def create_qa_notes(self):
        # After ensuring that the weblog directory is untarred/exists, create empty qa_notes.html file
        self.logger.info("Creating QA Notes file!")
        path = glob.glob(self.settings["destination_dir"] + "/products/pipeline-*/html")[0]
        os.chdir(path)
        qa_notes = open("qa_notes.html", "x")
        qa_notes.write("No QA notes found")
        os.chmod(qa_notes.name, 0o777)
        qa_notes.close()
        os.chdir("../../../")

    def move_subdirectories_to_qa_area(self, qa_path: str):
        # ignore non-casa related directories
        transfer_list = ["rawdata/", "working/", "products/"]

        if not pathlib.Path(qa_path).exists():
            os.makedirs(qa_path)
            os.chown(qa_path, vlapipe_id, vlapipe_id)
            os.chmod(qa_path, 0o2770)

        else:
            self.logger.info(f"Delivering to existing {qa_path}:")
            self.logger.info(os.listdir(qa_path))

        contents = glob.glob("*/", recursive=True)

        # deliver all directories before creating symlinks - ensures delivery if links fail
        for directory in contents:
            if directory in transfer_list:
                src = self.settings["destination_dir"] + "/" + directory
                dest = qa_path + "/" + directory
                shutil.move(src, dest)

        # link results back to intended destination
        for directory in contents:
            if directory in transfer_list:
                src = qa_path + "/" + directory
                os.system("chown -R 6000:6000 " + src)
                self.symlink_qa(directory, src)

    def symlink_qa(self, dir_name: str, src: pathlib.Path):
        ws_lustre_spool = self.settings["destination_dir"] + "/" + dir_name
        self.logger.info(f"Linking QA2 and workspaces parent directory for directory {dir_name}!")
        if not pathlib.Path(ws_lustre_spool).exists():
            pathlib.Path(ws_lustre_spool).symlink_to(src, target_is_directory=True)
            self.logger.info("Symlink Created!")
        else:
            self.logger.info(f"Directory {dir_name} already exists! Skipping symlink.")

    def convey(self):
        self.logger.info(f"RUNNING STANDARD {self.action.upper()}! ")
        self.delivery()
